import { Pool } from 'pg';
//
import IKDB from './../Prototype/IKDB';
//
global.POOL = null;
//
export default class IKDB_SQL extends IKDB {
  constructor(config) {
    super(config)
    if (config && config.database) { console.debug(`Using database: ${config.database} with config: ${config}`); }
    this.database = config.database;
    this.connection = null;
  }
  //Overwrites
  async connect() { return await (await this._getConnection()).connect(); }
  async cleanup() { return await (await this._getConnection()).release(); }
  async beginTransaction() { return await (await this._getConnection()).query('START TRANSACTION'); }
  async commit() { return await (await this._getConnection()).query('COMMIT'); }
  async rollback() { return await (await this._getConnection()).query('ROLLBACK'); }
  async runQuery(qry) { return await qry.run(await this._getConnection()); }
  //Public 
  async sanitize(val) { return (await this._getConnection()).getClient().escape(val); }

  /* private */
  async _getConnection() {
    if (!POOL) POOL = await this._newConnectionPool(this.config);
    if (this.connection) this.connection = await POOL.connect();
    return this.connection;
  }
  _newConnectionPool(config) {
    return new Promise((resolve, reject) => {
      const conn = new Pool({
        host: config.host, database: config.database,
        port: config.port, user: config.user,
        password: config.password, max: 20
      });
      resolve(conn);
    });
  }
}
